// Sample application to create an ISO YUV file from a raw YUV file
//
// Waqar Zia (waqar.zia@benq.com) 20 August, 2006
//

#include <stdio.h>
#include <string.h>
#include "IsoYUVFile.hpp"
#include "BufferManager.hpp"

using namespace ISOFile_Include_IsoFileClasses_hpp;
using namespace ISOFile_Include_BufferManager_hpp;
using namespace ISOFile_Include_IsoYUVFile_hpp;

void convert(char *infileName,char *outfileName,unsigned int framesToConvert,unsigned int width,unsigned int height,float frameRate);

int main (int argc, char **argv)
{
    unsigned int framesToConvert = 1000;
    unsigned int width = 176;
    unsigned int height = 144;
    float frameRate = 30.0;
    char infileName[1024];
    char outfileName[1024];
    const char *defaultInfile = "foreman_part_qcif.yuv";
    const char *defaultOutFile = "out.iso";
    
    if(argc == 7)
    {
        strcpy(infileName,argv[1]);
        strcpy(outfileName,argv[2]);
        framesToConvert = atoi(argv[3]);
        width = atoi(argv[4]);
        height = atoi(argv[5]);
        frameRate = atof(argv[6]);
    }
    else if(argc == 1)
    {
        strcpy(infileName,defaultInfile);
        strcpy(outfileName,defaultOutFile);
    }
    else
    {
        printf("Improper number of arguments!\n");
        return -1;
    }
    
    convert(infileName,outfileName,framesToConvert,width,height,frameRate);
    return 0;
}

void convert(char *infileName,char *outfileName,unsigned int framesToConvert,unsigned int width,unsigned int height,float frameRate)
{
   try 
    {
        FILE *infile = fopen(infileName,"rb");
        unsigned int frameSize = (height*width*3)/2;
        const unsigned int timeScale = 600;
        unsigned int frameDuration = (unsigned int)(((float)timeScale/frameRate)+0.5);
        FILE *outFile = fopen(outfileName,"wb");
        const unsigned int fixedHeaderSize = 16*1024;
        IsoYUVFile isoYUVFile(height,width,timeScale);
        unsigned int index;
        
        if(infile == NULL)
        {
            printf("Could not open infile %s!",infileName);
            return;
        }
        if(outFile == NULL)
        {
            printf("Could not open outFile %s!",outfileName);
            return;
        }

        fseek(outFile,fixedHeaderSize,SEEK_SET);

        Byte *frameBuffer = new Byte[frameSize];

        for(index = 0 ; index < framesToConvert ; index++)
        {
            if(index == 1)
                frameDuration*=2;
            int ret = (int)fread(frameBuffer,1,frameSize,infile);
            if(ret == 0)
            {
                printf("Infile terminated at %d frames\n",index);
                break;
            }
            isoYUVFile.addNewSampleInformation(frameSize,frameDuration);
            fwrite(frameBuffer,1,frameSize,outFile);
        }

        isoYUVFile.finalizeHeader(fixedHeaderSize);

        BufferManager fileBuffer;

        isoYUVFile.encodeAll(&fileBuffer);


        Byte *encodedBuffer;
        UInt32 count;

        count = fileBuffer.getBuffer(&encodedBuffer);
        
        fseek(outFile,0,SEEK_SET);

        fwrite(encodedBuffer,1,fixedHeaderSize,outFile);
       
        printf("Processed %d frames\n",index);

        delete [] frameBuffer;
        
        fclose(infile);
        
        fclose(outFile);

    }
    catch( char * str ) 
    {
        printf("Exception raised: %s\n",str);
    }
}

