// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//

#ifndef _ISOFILE_INCLUDE_CLASS_CONTAINER_HPP_
#define _ISOFILE_INCLUDE_CLASS_CONTAINER_HPP_

#include "Namespace.hpp"
#include "DataTypes.hpp"

DECLARE_NAMESPACE(ISOFile_Include_ClassContainer_hpp);

using namespace ISOFile_Include_IsoFileClasses_hpp;

/**
  *   Class ClassContainer, used to compose (not just aggregate) other Box class objects
  *   
  */
class ClassContainer
{
    public:
        ClassContainer();
        virtual ~ClassContainer();
        
    public:
        
      /**
        *   Create and add a new box with specified type (one of IsoBoxTypes)
        *
        *   @param  type    As one of IsoBoxTypes
        *
        *   @see    IsoBoxTypes
        *
        *   @return Box object of type
        */
        virtual Box* addNewBox(const char* type);
        
      /**
        *   Get first box of given type in collection, NULL if not found
        *
        *   @param  type    As one of IsoBoxTypes
        *
        *   @see    IsoBoxTypes
        *
        *   @return Box object of type, NULL if not found
        */
        Box* getBox(const char* type);
        
      /**
        *   Scans the contained boxes recursively for a given type. Returns the first occurance of the box encountered that matches the type.
        *   Always returns NULL for classes that cannot contain other classes according to ISO/IEC 14496-12:2005(E).
        *   
        *   @param  type    The type of box to look for
        *
        *   @return Pointer of Box if found, NULL otherwise.
        */
        Box* getBoxRecursive(const char* type);
      
      /**
        *   Remove and free all composition objects
        *
        */
        void removeAll();
      
      /**
        *   Encode all composition objects into the provided BufferManager object, all allocations required are done here internally
        *
        */
        void encodeAll(BufferManager *containerBuffer);
      
      /**
        *   Decode the data in provided BufferManager object, put decoded data in respective composed objects
        *
        */
        void decodeAll(BufferManager *containerBuffer);
      
      /**
        *   Estimate the overall size of all composition objects
        *
        */
        UInt32 estimateMemberSize();
        
    protected:
        Vector<Box *> boxes;

    private:
        void add(Box *box);
        Box* getNewBox(const char* type);
        
};

END_NAMESPACE(ISOFile_Include_ClassContainer_hpp);

#endif //_ISOFILE_INCLUDE_CLASS_CONTAINER_HPP_

