// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 29 May, 2006
//

#define _ISOFILE_CLASS_CONTAINER_CPP_

#include <string.h>
#include <stdio.h>
#include "IsoFileClasses.hpp"
#include "IsoBoxTypes.hpp"

using namespace ISOFile_Include_ClassContainer_hpp;
using namespace ISOFile_Include_IsoBoxTypes_hpp;

ClassContainer::ClassContainer()
{
}

ClassContainer::~ClassContainer()
{
    removeAll();
}
        
Box* ClassContainer::addNewBox(const char* type)
{
    Box *newBox = getNewBox(type);
    
    if(newBox == NULL)
        throw "ClassContainer::addNewBox: Cannot get box!";
    
    boxes[boxes.getSize()] = newBox;
    return newBox;
}

Box* ClassContainer::getBoxRecursive(const char* type)
{
    UInt32 uintType;

    Box::mapStringToUInt32((char*)type,&uintType);
    
    for(UInt32 index = 0 ; index < boxes.getSize() ; index++)
    {
        if(boxes[index]->type == uintType)
            return boxes[index];

        Box *foundBox = boxes[index]->getBoxRecursive(type);
        
		if(foundBox != NULL)
			return foundBox;
    }
    return NULL;
}

Box* ClassContainer::getBox(const char* type)
{
    UInt32 uintType;
    Box::mapStringToUInt32((char*)type,&uintType);
    
    for(UInt32 index = 0 ; index < boxes.getSize() ; index++)
        if(boxes[index]->type == uintType)
            return boxes[index];
        
    return NULL;
}

void ClassContainer::add(Box *box)
{
    boxes[boxes.getSize()] = box;
}

void ClassContainer::removeAll()
{
    for(UInt32 index = 0 ; index < boxes.getSize() ; index++)
    {
        delete boxes[index];
    }
    
   boxes.setSize(0);
}

void ClassContainer::encodeAll(BufferManager *containerBuffer)
{
    for(UInt32 index = 0 ; index < boxes.getSize() ; index++)
    {
        Byte *buffer;
        UInt32 size;
        boxes[index]->encode(&buffer,&size);
        containerBuffer->writeData(buffer,size);
    }
}

UInt32 ClassContainer::estimateMemberSize()
{
    UInt32 memberSize = 0;
    
    for(UInt32 index = 0 ; index < boxes.getSize() ; index++)
    {
        memberSize += boxes[index]->estimateSize();
    }

    return memberSize;
}

void ClassContainer::decodeAll(BufferManager *containerBuffer)
{
    Byte *buffer;
    UInt32 size;

    size = containerBuffer->getBuffer(&buffer);

    while(containerBuffer->getReadPointer() < size-1)
    {
		Box temp;
        UInt32 savedReadPointer = containerBuffer->getReadPointer();

        temp.decode(&(buffer[savedReadPointer]), size-savedReadPointer);

        char box_type[5];

        temp.mapUInt32ToString(&temp.type,box_type);
 
        Box *newBox = NULL;

        newBox = getNewBox(box_type);

        if(newBox == NULL)
            throw "ClassContainer::decodeAll: Cannot get box!";

        try
        {
            newBox->decode(&(buffer[savedReadPointer]),temp.size > 0 ? temp.size : size-savedReadPointer);
        }
        catch(int reqSize)
        {
            delete newBox;
            throw reqSize;
        }
        
        add(newBox);

        containerBuffer->setReadPointer(savedReadPointer+(temp.size > 0 ? temp.size : size-savedReadPointer));
        
    }
}

Box* ClassContainer::getNewBox(const char* type)
{
    if(strcmp(IsoBoxTypes::type_FileTypeBox,type) == 0)
        return new FileTypeBox();
    else if(strcmp(IsoBoxTypes::type_MovieBox,type) == 0)
        return new MovieBox();
    else if(strcmp(IsoBoxTypes::type_MediaDataBox,type) == 0)
        return new MediaDataBox();
    else if(strcmp(IsoBoxTypes::type_FreeSpaceBox,type) == 0)
        return new FreeSpaceBox();  
    else if(strcmp(IsoBoxTypes::type_MovieHeaderBox,type) == 0)
        return new MovieHeaderBox(0);
    else if(strcmp(IsoBoxTypes::type_TrackBox,type) == 0)
        return new TrackBox();
    else if(strcmp(IsoBoxTypes::type_TrackHeaderBox,type) == 0)
        return new TrackHeaderBox(0,0);
    else if(strcmp(IsoBoxTypes::type_MediaBox,type) == 0)
        return new MediaBox();
    else if(strcmp(IsoBoxTypes::type_MediaHeaderBox,type) == 0)
        return new MediaHeaderBox(0);
    else if(strcmp(IsoBoxTypes::type_HandlerBox,type) == 0)
        return new HandlerBox();
    else if(strcmp(IsoBoxTypes::type_MediaInformationBox,type) == 0)
        return new MediaInformationBox();
    else if(strcmp(IsoBoxTypes::type_VideoMediaHeaderBox,type) == 0)
        return new VideoMediaHeaderBox();
    else if(strcmp(IsoBoxTypes::type_DataInformationBox,type) == 0)
        return new DataInformationBox();
    else if(strcmp(IsoBoxTypes::type_DataEntryUrlBox,type) == 0)
        return new DataEntryUrlBox(0,0);
    else if(strcmp(IsoBoxTypes::type_DataEntryUrnBox,type) == 0)
        return new DataEntryUrnBox(0,0);
    else if(strcmp(IsoBoxTypes::type_DataReferenceBox,type) == 0)
        return new DataReferenceBox(0,0);
    else if(strcmp(IsoBoxTypes::type_SampleTableBox,type) == 0)
        return new SampleTableBox();
    else if(strcmp(IsoBoxTypes::type_TimeToSampleBox,type) == 0)
        return new TimeToSampleBox();
    else if(strcmp(IsoBoxTypes::type_CompositionOffsetBox,type) == 0)
        return new CompositionOffsetBox();
    else if(strcmp(IsoBoxTypes::type_SampleDescriptionBox,type) == 0)
        return new SampleDescriptionBox();
     else if(strcmp(IsoBoxTypes::type_SampleSizeBox,type) == 0)
        return new SampleSizeBox();
     else if(strcmp(IsoBoxTypes::type_SampleToChunkBox,type) == 0)
        return new SampleToChunkBox();
     else if(strcmp(IsoBoxTypes::type_ChunkOffsetBox,type) == 0)
        return new ChunkOffsetBox();
   else
        return NULL;
}

    
