// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//

#define _ISOFILE_ISO_YUV_FILE_CPP_

#ifdef WIN32
#define _CRT_SECURE_NO_DEPRECATE
#endif

#include <memory.h>
#include "IsoYUVFile.hpp"
#include "IsoBoxTypes.hpp"

using namespace ISOFile_Include_IsoYUVFile_hpp;
using namespace ISOFile_Include_IsoFileClasses_hpp;
using namespace ISOFile_Include_IsoBoxTypes_hpp;
    
IsoYUVFile::IsoYUVFile()
{
    accumulatedOffset = 0;
}

IsoYUVFile::IsoYUVFile(UInt32 height, UInt32 width, UInt32 timeScale)
{
    accumulatedOffset = 0;
    init(height,width,timeScale);
}

IsoYUVFile::~IsoYUVFile()
{
}

void IsoYUVFile::init(UInt32 height, UInt32 width, UInt32 timeScale, bool useSpecifiedTime, UInt32 specifiedTime)
{
    FileTypeBox *ftyp = (FileTypeBox *)this->addNewBox(IsoBoxTypes::type_FileTypeBox);

    Box::mapStringToUInt32("isom",&ftyp->major_brand);
    
    ftyp->minor_version = 0;

    MovieBox *moov = (MovieBox *)this->addNewBox(IsoBoxTypes::type_MovieBox);

    trackID = 0x1;

    unsigned_int32 currentTime = useSpecifiedTime ? specifiedTime : MovieHeaderBox::getCurrentISOFileFormatTime();

    MovieHeaderBox *mvhd = (MovieHeaderBox *)moov->addNewBox(IsoBoxTypes::type_MovieHeaderBox);

    mvhd->creation_time = currentTime;
    mvhd->modification_time = currentTime;
    mvhd->timescale = timeScale;
    mvhd->duration = 0;     //To be finalized
    mvhd->next_track_ID = trackID;

    TrackBox *trak = (TrackBox *)moov->addNewBox(IsoBoxTypes::type_TrackBox);
    TrackHeaderBox *tkhd = (TrackHeaderBox *)trak->addNewBox(IsoBoxTypes::type_TrackHeaderBox);

    tkhd->creation_time = currentTime;
    tkhd->modification_time = currentTime;
    tkhd->track_ID = trackID;
    tkhd->duration = (UInt32)-1;     //To be finalized
    tkhd->width = width << 16;
    tkhd->height = height << 16;

    MediaBox *mdia = (MediaBox *)trak->addNewBox(IsoBoxTypes::type_MediaBox);

    char *language = "zxx"; //ISO 639-2: No linguistic content
    MediaHeaderBox *mdhd = (MediaHeaderBox *)mdia->addNewBox(IsoBoxTypes::type_MediaHeaderBox);

    mdhd->creation_time = currentTime;
    mdhd->modification_time = currentTime;
    mdhd->timescale = timeScale;
    mdhd->duration = (UInt32)-1;     //To be finalized
    mdhd->setLanguageFromISOCode(language);

    char *handler = "vide";
    HandlerBox *hdlr = (HandlerBox *)mdia->addNewBox(IsoBoxTypes::type_HandlerBox);
    
    Box::mapStringToUInt32(handler, &hdlr->handler_type);
    hdlr->set_name(handler);

    MediaInformationBox *minf = (MediaInformationBox *)mdia->addNewBox(IsoBoxTypes::type_MediaInformationBox);
    
    VideoMediaHeaderBox *vmhd = (VideoMediaHeaderBox *)minf->addNewBox(IsoBoxTypes::type_VideoMediaHeaderBox);

    DataInformationBox *dinf = (DataInformationBox *)minf->addNewBox(IsoBoxTypes::type_DataInformationBox);

    DataReferenceBox *dref = (DataReferenceBox *)minf->addNewBox(IsoBoxTypes::type_DataReferenceBox);

    DataEntryUrnBox *urn = (DataEntryUrnBox *)dref->addNewBox(IsoBoxTypes::type_DataEntryUrnBox);

    urn->flags = 0x000001;

    SampleTableBox *stbl = (SampleTableBox *)minf->addNewBox(IsoBoxTypes::type_SampleTableBox);

    TimeToSampleBox *stts = (TimeToSampleBox *)stbl->addNewBox(IsoBoxTypes::type_TimeToSampleBox);

    CompositionOffsetBox *ctts = (CompositionOffsetBox *)stbl->addNewBox(IsoBoxTypes::type_CompositionOffsetBox);

    SampleDescriptionBox *stsd = (SampleDescriptionBox *)stbl->addNewBox(IsoBoxTypes::type_SampleDescriptionBox);

    VisualSampleEntry *visualSampleEntry = (VisualSampleEntry *)stsd->addNewSampleEntry(stsd->handler_types[1]);

    visualSampleEntry->width = width;

    visualSampleEntry->height = height;

    visualSampleEntry->set_compressorname((string)"j420");

    SampleSizeBox *stsz = (SampleSizeBox *)stbl->addNewBox(IsoBoxTypes::type_SampleSizeBox);

    SampleToChunkBox *stsc = (SampleToChunkBox *)stbl->addNewBox(IsoBoxTypes::type_SampleToChunkBox);

    ChunkOffsetBox *stco = (ChunkOffsetBox *)stbl->addNewBox(IsoBoxTypes::type_ChunkOffsetBox);

    FreeSpaceBox *free = (FreeSpaceBox *)this->addNewBox(IsoBoxTypes::type_FreeSpaceBox);

    MediaDataBox *mdat = (MediaDataBox *)this->addNewBox(IsoBoxTypes::type_MediaDataBox);

    mdat->size = 0; //Till end of file

    vmhd=NULL;dinf=NULL;stts=NULL;ctts=NULL;stsz=NULL;stsc=NULL;stco=NULL;free=NULL;//For the time being, set to NULL in the end to supress GCC warnings
}

void IsoYUVFile::addNewSampleInformation(UInt32 size, UInt32 timeDelta, bool useSpecifiedTime, UInt32 specifiedTime)
{
    unsigned_int32 currentTime = useSpecifiedTime ? specifiedTime : MovieHeaderBox::getCurrentISOFileFormatTime();

    MovieHeaderBox *mvhd = (MovieHeaderBox *)getBoxRecursive(IsoBoxTypes::type_MovieHeaderBox);

    mvhd->modification_time = currentTime;

    TrackHeaderBox *tkhd = (TrackHeaderBox *)getBoxRecursive(IsoBoxTypes::type_TrackHeaderBox);

    tkhd->modification_time = currentTime;

    MediaHeaderBox *mdhd = (MediaHeaderBox *)getBoxRecursive(IsoBoxTypes::type_MediaHeaderBox);

    mdhd->modification_time = currentTime;
    
    TimeToSampleBox *stts = (TimeToSampleBox *)getBoxRecursive(IsoBoxTypes::type_TimeToSampleBox);

    stts->add_sample_delta(timeDelta);

    CompositionOffsetBox *ctts = (CompositionOffsetBox *)getBoxRecursive(IsoBoxTypes::type_CompositionOffsetBox);

    ctts->add_sample_offset(0);

    SampleSizeBox *stsz = (SampleSizeBox *)getBoxRecursive(IsoBoxTypes::type_SampleSizeBox);

    stsz->add_sample_size(size);
    
    ChunkOffsetBox *stco = (ChunkOffsetBox *)getBoxRecursive(IsoBoxTypes::type_ChunkOffsetBox);

    stco->add_chunk_offset(accumulatedOffset);

    accumulatedOffset += size;
}

void IsoYUVFile::finalizeHeader(UInt32 requiredSize)
{
    MovieHeaderBox *mvhd = (MovieHeaderBox *)getBoxRecursive(IsoBoxTypes::type_MovieHeaderBox);
    
    TimeToSampleBox *stts = (TimeToSampleBox *)getBoxRecursive(IsoBoxTypes::type_TimeToSampleBox);
    
    UInt32 mediaDuration = (UInt32)((double)stts->getCumulativeSampleDelta()/(double)mvhd->timescale + 0.5);

    mvhd->duration = mediaDuration;

    TrackHeaderBox *tkhd = (TrackHeaderBox *)getBoxRecursive(IsoBoxTypes::type_TrackHeaderBox);

    tkhd->duration = mediaDuration;

    MediaHeaderBox *mdhd = (MediaHeaderBox *)getBoxRecursive(IsoBoxTypes::type_MediaHeaderBox);

    mdhd->duration = mediaDuration;

    UInt32 totalHeaderSize = estimateMemberSize();

    if(requiredSize > 0 && requiredSize < totalHeaderSize)
        throw "IsoYUVFile::finalizeHeader: Not possible to scale header to required size!";

    if(requiredSize > 0 && requiredSize > totalHeaderSize)
    {
        UInt32 sizeToStuff = requiredSize-totalHeaderSize;
        
        Byte *temp = new Byte[sizeToStuff];

        memset(temp,0xFF,sizeToStuff*sizeof(Byte));
        
        FreeSpaceBox *free = (FreeSpaceBox *)getBoxRecursive(IsoBoxTypes::type_FreeSpaceBox);

        free->addData(temp,sizeToStuff);

        totalHeaderSize += sizeToStuff;
        
        if(sizeToStuff > 1)
            delete [] temp;
        else
            delete temp;
    }

    updateChunkOffset(totalHeaderSize);
}

void IsoYUVFile::updateChunkOffset(UInt32 offsetToAdd)
{
    ChunkOffsetBox *stco = (ChunkOffsetBox *)getBoxRecursive(IsoBoxTypes::type_ChunkOffsetBox);

    UInt32 oldOffset = stco->chunk_offset[0];

    for(UInt32 index = 0 ; index < stco->entry_count ; index++)
    {
        stco->chunk_offset[index] -= oldOffset;
        stco->chunk_offset[index] += offsetToAdd;
    }
}

